<?php

namespace Payshop\Core\Block;

use Magento\Sales\Model\OrderRepository;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Payshop\Core\Validation\ConfirmCardPayment as ValidationConfirmCardPayment;

class ConfirmCardPayment extends Template
{
    /**
     * @var OrderRepository
     */
    private OrderRepository $orderRepository;

    /**
     * ConfirmCardPayment constructor.
     * 
     * @param Context $context
     * @param OrderRepository $orderRepository
     */
    public function __construct(
        Context $context,
        OrderRepository $orderRepository
    ) {
        parent::__construct($context);
        $this->orderRepository = $orderRepository;
    }

    /**
     * Get order id from request
     * 
     * @return int
     */
    public function getOrderId(): int
    {
        $orderId = $this->getRequest()->getParam('order_id');
        return $orderId;
    }

    /**
     * Get order by id
     * 
     * @return OrderInterface
     */
    public function getOrder(): OrderInterface
    {
        $orderId = $this->getOrderId();

        return $this->orderRepository->get($orderId);
    }

    /**
     * Get order total value
     * 
     * @return float
     */
    public function getOrderValue(): float
    {
        $order = $this->getOrder();
       
        return number_format($order->getGrandTotal(), 2, '.', '');
    }

    /**
     * Check if order is payed with card
     * 
     * @return bool
     */
    public function isCard(): bool
    {
        $order = $this->getOrder();

        return ValidationConfirmCardPayment::isCard($order);
    }

    /**
     * Check if order is payed in two steps
     * 
     * @return bool
     */
    public function isPaymentInTwoSteps(): bool
    {
        $order = $this->getOrder();

        return ValidationConfirmCardPayment::isPaymentInTwoSteps($order);
    }

    /**
     * Check if order efective value is confirmed
     * 
     * @return bool
     */
    public function isPaymentConfirmed(): bool
    {
        $order = $this->getOrder();

        return ValidationConfirmCardPayment::isPaymentConfirmed($order);
    }

    /**
     * Check if order is in processing status
     * 
     * @return bool
     */
    public function isProcessingStatus(): bool
    {
        $order = $this->getOrder();

        return ValidationConfirmCardPayment::isProcessingStatus($order);
    }

    /**
     * Get de limit date to confirm payment
     * 
     * @return string
     */
    public function getDateLimitToConfirmPayment(): string
    {
        $order = $this->getOrder();

        return ValidationConfirmCardPayment::getDateLimitToConfirmPayment($order)->format('d/m/Y H:i:s');
    }

    /**
     * Check if limit date to confirm payment is expired
     * 
     * @return bool
     */
    public function isDateLimitToConfirmPaymentExpired(): bool
    {
        $order = $this->getOrder();

        return ValidationConfirmCardPayment::isDateLimitToConfirmPaymentExpired($order);
    }
}
