<?php

namespace Payshop\Core\Controller\Card;

use Magento\Quote\Model\Quote;
use Magento\Checkout\Model\Type\Onepage;
use Magento\Quote\Api\CartManagementInterface;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\Controller\Result\RedirectFactory;

class Process implements HttpGetActionInterface
{
    /**
     * @var Quote
     */
    protected Quote $quote;
    
    /**
     * @var Onepage
     */
    protected Onepage $checkoutSession;
    
    /**
     * @var RedirectFactory
     */
    protected RedirectFactory $redirectFactory;
    
    /**
     * @var ManagerInterface
     */
    protected ManagerInterface $messageManager;
    
    /**
     * @var CartManagementInterface
     */
    protected CartManagementInterface $cartManagement;

    /**
     * Process constructor.
     * 
     * @param Quote $quote
     * @param Onepage $checkoutSession
     * @param RedirectFactory $redirectFactory
     * @param ManagerInterface $messageManager
     * @param CartManagementInterface $cartManagement
     */
    public function __construct(
        Quote $quote,
        Onepage $checkoutSession,
        RedirectFactory $redirectFactory,
        ManagerInterface $messageManager,
        CartManagementInterface $cartManagement
    )
    {
        $this->quote = $quote;
        $this->checkoutSession = $checkoutSession;
        $this->redirectFactory = $redirectFactory;
        $this->messageManager = $messageManager;
        $this->cartManagement = $cartManagement;
    }

    /**
     * Check if mbway payment is confirmed
     * 
     * @return \Magento\Framework\Controller\Result\Redirect
     */
    public function execute()
    {
        $resultRedirect = $this->redirectFactory->create();

        try {
            $this->quote = $this->checkoutSession->getQuote();
            $this->quote->getPayment()->setMethod('payshop_card');
            $this->cartManagement->placeOrder($this->quote->getId());

            return $resultRedirect->setPath('checkout/onepage/success');
        } catch(\Throwable $e) {
            $this->messageManager->addErrorMessage($e->getMessage());

            return $resultRedirect->setPath('checkout', ['_fragment' => 'payment']);
        }
    }
}