<?php

namespace Payshop\Core\Controller\Mbway;

use Magento\Sales\Model\Order;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\Action\HttpGetActionInterface;

class Check implements HttpGetActionInterface
{
    /**
     * @var Order
     */
    private Order $order;

    /**
     * @var RequestInterface RequestInterface
     */
    private RequestInterface $request;

    /**
     * @var JsonFactory
     */
    protected JsonFactory $resultJsonFactory;

    /**
     * Check constructor.
     * 
     * @param Order $order
     * @param RequestInterface $request
     * @param JsonFactory $resultJsonFactory
     */
    public function __construct(
        Order $order,
        RequestInterface $request,
        JsonFactory $resultJsonFactory
    ) {
        $this->order = $order;
        $this->request = $request;
        $this->resultJsonFactory = $resultJsonFactory;
    }

    /**
     * Check if mbway payment is confirmed
     * 
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultJsonFactory->create();

        $orderId = $this->request->getParam('orderId');
        $order = $this->order->loadByAttribute('entity_id', $orderId);
        $orderState = $order->getState();

        if ($orderState == Order::STATE_PROCESSING) {
            return $response->setData([
                'status' => 'paid'
            ]);
        } 
        
        if ($orderState == Order::STATE_CANCELED) {
            return $response->setData([
                'status' => 'declined'
            ]);
        } 

        return $response->setData(['status' => 'waiting']);
    }
}