<?php

namespace Payshop\Core\Gateway\Authorize\Response;

use Magento\Sales\Model\Order\Payment;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;

class SaveTransactionInformationsHandler implements HandlerInterface
{
    /**
     * @inheritDoc
     */
    public function handle(array $handlingSubject, array $response)
    {
        if (!isset($handlingSubject['payment'])
            || !$handlingSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }

        $instrument = $response['instrument'];

        $paymentDataObject = $handlingSubject['payment'];
        $payment = $paymentDataObject->getPayment();

        $this->setTransactionInformations($payment, $instrument);
        $this->saveChargeAndInstrument($payment, $instrument);
        $this->savePaymentInTwoSteps($payment, $instrument);
    }

    /**
     * Save transaction informations
     * 
     * @param Payment $payment
     * @param array $instrument
     * @return void
     */
    private function setTransactionInformations(Payment $payment, array $instrument): void
    {
        $payment->setIsTransactionPending(false);
        $payment->setIsTransactionClosed(true);
        $payment->setTransactionId($instrument['id']);
    }

    /**
     * Save charge and instrument ID
     * 
     * @param Payment $payment
     * @param array $instrument
     * @return void
     */
    private function saveChargeAndInstrument(Payment $payment, array $instrument): void
    {
        $payment->setAdditionalInformation(
            'instrument_id',
            $instrument['id'],
        );

        $payment->setAdditionalInformation(
            'charge_id',
            $instrument['charge']['id'],
        );
    }

    /**
     * Save payment in two steps
     * 
     * @param Payment $payment
     * @param array $instrument
     * @return void
     */
    private function savePaymentInTwoSteps(Payment $payment, array $instrument): void
    {
        $paymentInTwoSteps = $instrument['capture_on_creation'] == false;

        $payment->setAdditionalInformation(
            'payment_in_two_steps',
            $paymentInTwoSteps ? 'Sim' : 'Não',
        );
    }
}
