<?php

namespace Payshop\Core\Gateway\Initialize\Http\Client;

use Payshop\Core\Action\CreateCharge;
use Payshop\Core\Action\CreateInstrument;
use Magento\Payment\Gateway\Http\ClientInterface;
use Magento\Payment\Gateway\Http\TransferInterface;

class PayshopTransaction implements ClientInterface
{
    /**
     * @var CreateCharge
     */
    private CreateCharge $createCharge;

    /**
     * @var CreateInstrument
     */
    private CreateInstrument $createInstrument;

    /**
     * PayshopTransaction constructor.
     * 
     * @param CreateCharge $createCharge
     * @param CreateInstrument $createInstrument
     */
    public function __construct(
        CreateCharge $createCharge,
        CreateInstrument $createInstrument
    ) {
        $this->createCharge = $createCharge;
        $this->createInstrument = $createInstrument;
    }

    /**
     * @inheritDoc
     */
    public function placeRequest(TransferInterface $transferObject)
    {
        $requestData = $transferObject->getBody();

        try {
            $this->checkCurrency($requestData['charge']['currency']);

            $charge = $this->createCharge->execute($requestData['charge']);

            $instrumentRequest = $requestData['instrument'] + ["charge" => $charge['id']];
            $instrument = $this->createInstrument->execute($instrumentRequest);

            return compact('instrument');
        } catch (\Throwable $th) {
            return [
                'status' => 'error',
                'code' => $th->getMessage()
            ];
        }
    }

    /**
     * Check if currency is EUR
     * 
     * @param string $currency
     * @return void
     * @throws \Exception
     */
    private function checkCurrency(string $currency): void
    {
        if ($currency != 'EUR') {
            throw new \Exception('400003');
        }
    }
}
