<?php

namespace Payshop\Core\Gateway\Initialize\Response;

use Magento\Sales\Model\Order\Payment;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;

class SaveTransactionInformationsHandler implements HandlerInterface
{
    /**
     * @inheritDoc
     */
    public function handle(array $handlingSubject, array $response)
    {
        if (!isset($handlingSubject['payment'])
            || !$handlingSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }

        $instrument = $response['instrument'];

        $paymentDataObject = $handlingSubject['payment'];
        $payment = $paymentDataObject->getPayment();

        $this->setTransactionInformations($payment, $instrument);
        $this->saveChargeAndInstrument($payment, $instrument);
        $this->saveReference($payment, $instrument);
    }

    /**
     * Save transaction informations
     * 
     * @param Payment $payment
     * @param array $instrument
     * @return void
     */
    private function setTransactionInformations(Payment $payment, array $instrument): void
    {
        $payment->setIsTransactionPending(true);
        $payment->setIsTransactionClosed(false);
        $payment->setTransactionId($instrument['id']);
        $payment->addTransaction(Transaction::TYPE_PAYMENT);
    }

    /**
     * Save charge and instrument ID
     * 
     * @param Payment $payment
     * @param array $instrument
     * @return void
     */
    private function saveChargeAndInstrument(Payment $payment, array $instrument): void
    {
        $payment->setAdditionalInformation(
            'instrument_id',
            $instrument['id'],
        );

        $payment->setAdditionalInformation(
            'charge_id',
            $instrument['charge']['id'],
        );
    }

    /**
     * Save reference
     * 
     * @param Payment $payment
     * @param array $instrument
     * @return void
     */
    private function saveReference(Payment $payment, array $instrument): void
    {
        $isNotPayshopReference = $payment->getMethod() !== 'payshop_payshop_reference';
        $isNotMultibancoReference = $payment->getMethod() !== 'payshop_multibanco_reference';

        if ($isNotPayshopReference && $isNotMultibancoReference) {
            return;
        }

        foreach ($instrument['reference']['fields'] as $field) {
            $payment->setAdditionalInformation(
                $field['field'],
                $field['value'],
            );
        }
    }
}
